%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CalculateFits
% 
% Reference: Fetal/Juvenile Tumor Initiation (2010)
% Authors: Kini, L.G., Herrero-Jimenez P., Guitterez Jr E., Hensle, D.,
%    Kogel J., Kusko R., Rexer, K., Kurzweil, R., Gostjeva, E., 
%    Morgenthaler, S., and Thilly WG.
% Contact: Lohith Kini <lkini (AT) mit DOT edu>
% Version: 5.0
% Last modified: March 22, 2010
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [ CAL ] = CalcCAL( amax,m,n, F,f,mu,Ri,RA,Ri_deciles,RA_deciles )
%CalcCAL Calculates CAL(g) for given model parameters based on model
%   equation in Kini et al (2010).
%
%   Input variables are model parameters:
%    amax, m, n and fit parameters F, f, mu, R_initiation and R_promotion

nRi_deciles = length(Ri_deciles);
nRA_deciles = length(RA_deciles);

Ri_deciles_avg = mean(Ri_deciles);
RA_deciles_avg = mean(RA_deciles);
Ri_deciles = Ri_deciles/Ri_deciles_avg;
RA_deciles = RA_deciles/RA_deciles_avg;
[A,G, Ri_dec, RA_dec] = ndgrid(amax-floor(amax):1:amax,0:1:50,Ri_deciles,RA_deciles);

if(m >= 2)
    Vobs = n*((Ri*Ri_dec).^n) .* ((RA*RA_dec).^m) .* A.^(n-1) .* 2.^G .* (2*(G-A+1).^(m-1) - (G-A).^(m-1)) .* exp(- ((RA*RA_dec).^m) .* (G-A).^(m-1) .* 2.^(G-A) );
else
    Vobs = n*log(2)*((Ri*Ri_dec).^n) .* ((RA*RA_dec).^m) .* A.^(n-1) .* 2.^G .* exp(- ((RA*RA_dec).^m) .* (G-A).^(m-1) .* 2.^(G-A) );
end

mask = zeros(floor(amax)+1,51, nRi_deciles, nRA_deciles);

for g = 0:1:50
    for a = 1:1:min(floor(amax),g)+1
        mask(a,g+1,:,:) = 1;
    end
end

Vobs = Vobs .* mask;

Vobs = sum(Vobs,1);
Pobs = 1 - exp(-Vobs); 
Pobs(find(Pobs == 0)) = 1E-100;

int_Vobs = cumsum(Vobs);

bottom_term = exp(-1/f * int_Vobs);
top_term = Pobs .* bottom_term;

sum_top_term = sum(top_term,3);
sum_top_term = squeeze(sum(sum_top_term,4));

sum_bottom_term = sum(bottom_term,3);
sum_bottom_term = squeeze(sum(sum_bottom_term,4));


CAL = F*sum_top_term ./ (F*sum_bottom_term + nRi_deciles*nRA_deciles*(1-F));

% Optional script to plot CAL(g) per 100,000 from g = [0 .. 50]
% semilogy(0:1:50,CAL*1E5,'s-')
% grid on
% legend('CAL(g)')

end
