%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CalculateFits
% 
% Reference: Fetal/Juvenile Tumor Initiation (2010)
% Authors: Kini, L.G., Herrero-Jimenez P., Guitterez Jr E., Hensle, D.,
%    Kogel J., Kusko R., Rexer, K., Kurzweil, R., Gostjeva, E., 
%    Morgenthaler, S., and Thilly WG.
% Contact: Lohith Kini <lkini (AT) mit DOT edu>
% Version: 5.0
% Last modified: March 22, 2010
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [ handles ] = CalculateFits( handles )
%CALCULATEFITS Calculates all the best fits for CancerFit
%   handles - Contains the GUI (Graphical User Interface) parameters
%   Note: R_init = {\Pi_{m} R_initiation_m } ^ 1/m and
%         R_prom = {\Pi_{n} R_promotion_n } ^ 1/n 

disp('Initializing CancerFit Fit calculation engine ... ')
start_time = cputime;

disp('Initializing Fit Parameters ...')
% Capture all equation parameter ranges and number of iterations
nF = handles.nF;
Fstart = handles.Fstart;
Fend = handles.Fend;
if(Fstart == 0)
    Fstart = 1E-4;
end

nf = handles.nsf;
sfstart = handles.sfstart;
sfend = handles.sfend;
if(sfstart == 0)
    sfstart = 1E-4;
end

nmu = handles.nmu;
mustart = handles.mustart;
muend = handles.muend;

nRi = handles.nRi;
Ristart = handles.Ristart;
Riend = handles.Riend;

nRA = handles.nRA;
RAstart = handles.RAstart;
RAend = handles.RAend;

disp('Initializing Model Parameters ...')
% Correct INC since INC(t) is input as "incidence per capita (100,000)"
INC = handles.INC(8:end)*1E-5;
nonzero_inds = find(INC);

amax = handles.amax;
n = handles.n;
m = handles.m;

disp('Computing Fits ...')

% Calculate the iterations for all 5 fit variables
if(nF == 1)
    Fseries = Fstart:1:Fstart;
else
    Fseries = Fstart:(Fend-Fstart)/(nF-1):Fend;
end

if(nf == 1)
    sfseries = sfstart:1:sfstart;
else
    sfseries = sfstart:(sfend-sfstart)/(nf-1):sfend;
end

if(nmu == 1)
    museries = mustart:1:mustart;
else
    museries = mustart:(muend-mustart)/(nmu-1):muend;
end

[int_Vobs_large,F,f,mu] = ndgrid(0:1:50,Fseries,sfseries,museries);
[tmpF,tmpf,tmpmu] = ndgrid(Fseries,sfseries,museries);

% Create deciles for Rinit and Rprom
%nRi_deciles = 10;
%nRA_deciles = 10;

nRi_deciles = handles.nstrata;
nRA_deciles = handles.nstrata;

Ri_deciles = [handles.startstrata:(handles.endstrata-1)/(handles.nstrata-1):handles.endstrata];
RA_deciles = [handles.startstrata:(handles.endstrata-1)/(handles.nstrata-1):handles.endstrata];
Ri_deciles_avg = mean(Ri_deciles);
RA_deciles_avg = mean(RA_deciles);
Ri_deciles = Ri_deciles/Ri_deciles_avg;
RA_deciles = RA_deciles/RA_deciles_avg;

% Create g (stem cell doublings) and a variables
% Approximate a as a discrete sequence up to amax
[A,G, Ri_dec, RA_dec] = ndgrid(amax-floor(amax):1:amax,0:1:50,Ri_deciles,RA_deciles);

% Age groups
age_x = [17.5:5:102.5];

% Age - 19
age_x_m = [17.5-19:5:102.5-19];

% Large array of P_obs(g) as a function of F, f, mu
Pobs_large = zeros(51,nF,nf,nmu);
% Large array of CAL(t) as a function of F, f, mu
CAL_interp = zeros(18,nF,nf,nmu);
% Large repeated array of INC(t) repeated for each iteration of
%     F, f and mu
INC_large = repmat(INC', [1 nF nf nmu]);

% Error is calcuated as 
%     e = {1/18 * sum_{t=17.5}^{102.5} (log_10[CAL(t)] - log_10[INC(t)])^2 }^1/2

% Error_terms e for each iteration of F, f, mu
error_term = zeros(nF,nf,nmu);
error_reshaped = zeros(nF*nf*nmu,1);
% e^2 for each iteration of F, f, mu
error_sq_term = zeros(nF,nf,nmu);

num_fits = handles.num_fits;

% Initialize the top fits array which contains error, F,f,mu,R_init,R_prom
%    for each fit
top_fits = 1E10*ones(num_fits,6);

% Temporary fits array to merge after each iteration of {R_init,R_prom}
merged_fits = zeros(2*num_fits,6);

x = 0:1:50;

h=waitbar(0,'Fit Calculation Progress ... 0%');
set(h,'windowstyle','modal');

if (nRi == 1)
    Riend = Ristart;
end
if (nRA == 1)
    RAend = RAstart;
end

% Iterations for R_init and R_prom are in logarithmic steps if more than
%     a factor of 1000 apart and in linear steps if less
if(abs(log10(Ristart/Riend)) > 3)
    for i = 1:nRi
        Riseries(i) = 10^((log10(Riend)-log10(Ristart))/(nRi-1)*(i-1) + log10(Ristart));
    end
else
    Riseries = Ristart:(Riend-Ristart)/(nRi-1):Riend;
end

if(abs(log10(RAstart/RAend)) > 3)
    for i = 1:nRA
        RAseries(i) = 10^((log10(RAend)-log10(RAstart))/(nRA-1)*(i-1) + log10(RAstart));
    end
else
    RAseries = RAstart:(RAend-RAstart)/(nRA-1):RAend;
end

% Create temporary copies of each series
[G_tmp, Ri_dec_tmp, RA_dec_tmp,F_large,f_large,mu_large] = ndgrid(0:1:50,Ri_deciles,RA_deciles,Fseries,sfseries,museries);

% Store iterations to run through into GUI handles
handles.sfseries = sfseries;
handles.Fseries = Fseries;
handles.museries = museries;
handles.Riseries = Riseries;
handles.RAseries = RAseries;


frac_counter = 0;

%%%%%%%%%% ALGORITHM for Computing CANCERFIT Top Fits %%%%%%%%%% 
% For each R_init in R_init iteration series 
%     For each R_prom in R_prom iteration series 
%         Compute V_obs for each g from 0..50 and a from 0..amax
%             and approximate integral as a sum from a from to 0 up to
%             min(g,amax)
%         Compute P_obs = 1 - exp(-Vobs)
%         Compute integral of V_obs from 0 to g by sum approximation
%         Calculate CAL(g) = coefficient * P_obs
%
%         Since each matrix is search array over F, f, and mu, compute
%             the error between CAL(g) and INC(g) for each F, f, mu by
%             interpolation of CAL(g) to find value of CAL at t \in [17.5
%             .. 102.5], which in g domain is g \in [a_max .. amax + mu *
%             (102.5 - 17.5)] by out g to t linear transform.
%
%         Pick the top num_fit best fits and merge/sort with top_fits
%         variable
%     end For
% end For
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

for Ri = Riseries
    pause(0.2)
    frac = frac_counter/nRi;
    frac_counter = frac_counter+1;
    waitbar(frac,h,strcat('Fit Calculation Progress ... ',num2str(floor(frac*100)),'%'));
    for RA = RAseries
        if(m >= 2)
            Vobs = n*((Ri*Ri_dec).^n) .* ((RA*RA_dec).^m) .* A.^(n-1) .* 2.^G .* (2*(G-A+1).^(m-1) - (G-A).^(m-1)) .* exp(- ((RA*RA_dec).^m) .* (G-A).^(m-1) .* 2.^(G-A) );
        else
            Vobs = n*log(2)*((Ri*Ri_dec).^n) .* ((RA*RA_dec).^m) .* A.^(n-1) .* 2.^G .* exp(- ((RA*RA_dec).^m) .* (G-A).^(m-1) .* 2.^(G-A) );
        end
        
        mask = zeros(floor(amax)+1,51, nRi_deciles, nRA_deciles);

        for g = 0:1:50
            for a = 1:1:min(floor(amax),g)+1
                mask(a,g+1,:,:) = 1;
            end
        end
%         size(Vobs)
%         size(mask)
        Vobs = Vobs .* mask;
        
        Vobs = squeeze(sum(Vobs,1));
        Pobs = 1 - exp(-Vobs);        
        Pobs(find(Pobs == 0)) = 1E-100;
        Pobs_tmp = ones(51,nRi_deciles,nRA_deciles,1,1,1);
        Pobs_tmp(:,:,:,1,1,1) = Pobs;
        Pobs_large = repmat(Pobs_tmp,[1 1 1 nF nf nmu]);
        
        

        int_Vobs = cumsum(Vobs,1);
        int_Vobs_tmp = ones(51,nRi_deciles,nRA_deciles,1,1,1);
        int_Vobs_tmp(:,:,:,1,1,1) = int_Vobs(:,:,:);
        int_Vobs_large = repmat(int_Vobs_tmp,[1 1 1 nF nf nmu]);        
             
        bottom_term = exp(-1/f_large .* int_Vobs_large);
        top_term = Pobs_large .* bottom_term;
        
        sum_top_term = sum(top_term,2);
        sum_top_term = sum(sum_top_term,3);
        
        sum_bottom_term = sum(bottom_term,2);
        sum_bottom_term = sum(sum_bottom_term,3);
        
        sum_top_term2 = zeros(51,nF,nf,nmu);
        sum_bottom_term2 = zeros(51,nF,nf,nmu);
        for i_g = 1:51
            sum_top_term2(i_g,:,:,:) = sum_top_term(i_g,1,1,:,:,:);
            sum_bottom_term2(i_g,:,:,:) = sum_bottom_term(i_g,1,1,:,:,:);
        end

        Pobs_large = F .* sum_top_term2 ./ (F .* sum_bottom_term2 + nRi_deciles*nRA_deciles*(1-F));
        
        is = 1;
        
        if(nmu == 1)            
            for mu_val = mustart                   
                age = age_x_m*mu_val+amax;            
                y = Pobs_large(:,:,:,is);
                yi = interp1(x',y,age);
                CAL_interp(:,:,:,is) = yi;
                error_sq_term(:,:,is) = sum((log10(mu_val*CAL_interp(nonzero_inds,:,:,is) ./ INC_large(nonzero_inds,:,:,is))).^2,1);
                if(~isreal(error_sq_term))
                    'break';
                end
                is = is + 1;
            end 
        else
            for mu_val = mustart:(muend-mustart)/(nmu-1):muend                   
                age = age_x_m*mu_val+amax;            
                y = Pobs_large(:,:,:,is);
                yi = interp1(x',y,age);
                CAL_interp(:,:,:,is) = yi;
                error_sq_term(:,:,is) = sum((log10(mu_val*CAL_interp(nonzero_inds,:,:,is) ./ INC_large(nonzero_inds,:,:,is))).^2,1);
                if(~isreal(error_sq_term))
                    'break';
                end
                is = is + 1;
            end 
        end
        
        error_term = (error_sq_term/18).^0.5;        
        error_reshaped = reshape(error_term, [nF*nf*nmu 1]);
        error_term = zeros(nF,nf,nmu);
        [error_reshaped,ix] = sort(error_reshaped);

        % Take top num_fit best fits
        top_fits_tmp(:,1) = error_reshaped(1:num_fits,1);
        top_fits_tmp(:,2) = tmpF(ix(1:num_fits));
        top_fits_tmp(:,3) = tmpf(ix(1:num_fits));
        top_fits_tmp(:,4) = tmpmu(ix(1:num_fits));
        top_fits_tmp(:,5) = Ri;
        top_fits_tmp(:,6) = RA;       
        
        % Merge/sort with previous best fits
        merged_fits(1:num_fits,:) = top_fits_tmp;
        merged_fits(num_fits+1:2*num_fits,:) = top_fits;
        [sorted_merged_fits,ix] = sort(merged_fits,1);
        sorted_merged_fits(:,2:6) = merged_fits(ix(:,1),2:6);
        top_fits = sorted_merged_fits(1:num_fits,:);
    end
end
close(h);

% Given the best num_fits, re-compute CAL(g) for g = [0 .. 50] for each fit
CAL = zeros(num_fits,51);
for is = 1:num_fits
    % Re-Calculate CAL(g) from amax,m,n,F,f,mu,R_init,R_prom
    CAL(is,:) = CalcCAL( amax,m,n, top_fits(is,2),top_fits(is,3),top_fits(is,4),top_fits(is,5),top_fits(is,6), Ri_deciles, RA_deciles );
end

% If GUI results table is full, start from row 1
if(handles.curr_row == 11)
    handles.top_fits(10,1:num_fits,1:6) = top_fits;
    handles.top_fits(10,1:num_fits,7) = m;
    handles.top_fits(10,1:num_fits,8) = n;
    handles.top_fit_vals(10,1:num_fits,:) = CAL;
    handles.past_INC(10,:) = handles.INC;
else
    handles.top_fits(handles.curr_row,1:num_fits,1:6) = top_fits;
    handles.top_fits(handles.curr_row,1:num_fits,7) = m;
    handles.top_fits(handles.curr_row,1:num_fits,8) = n;
    handles.top_fit_vals(handles.curr_row,1:num_fits,:) = CAL;
    handles.past_INC(handles.curr_row,:) = handles.INC;
end

disp('Elapsed time for FIT Calculation ...')
cputime - start_time
end
